c.. Birth and death simulation of step function on [0,leng]
c   with Poisson (lambda) number of steps, uniformly distributed
c   on interval, and steps i.i.d. gamma(alpha,beta) 
c   with normal or poisson process likelihood with data y.

c.. step positions are sp(j), j = 1,...,k
c   and height of step between sp(j) and sp(j+1) is sh(j)

	parameter(kmax=30,nmax=500,ng=100)
	character*18 word,base,prfx
	character*30 wordl
	character sep
	character*80 line
	character*6 num
	integer sweep,seed
	real leng,lambda,logratio,logabcon
	real*8 dlgama
	logical qoptbd,qtab,qq,qbeta,qalpha,q1cp,qhar,qfix
	logical qieee,ieee_chk
	dimension sp(0:kmax+1),sh(0:kmax)
	dimension p(0:kmax),b(0:kmax),d(0:kmax)
	dimension x(ng),xav(ng),t(ng)

	dimension y(0:nmax),nct(100)
	common/datblk/y,nct,leng
c**
c.. options: in B'ka paper, power=1,conh=1,fac unused, qoptbd=T
	data fac/1.0/,power/1.0/,conh/1.0/
	data qoptbd/.true./,qtab/.false./
	data tp/6.283185/

	sep = "/"

	if(iargc().eq.0) then
	    write(0,*) "usage: cpt -ns[10] -n[1000] -nb[100]",
     &		" -a[1.0] -b[200.0] [-t] -seed[0] [-1cp] -freq[1]",
     &		" -ki[0] [-fix] {datafile}"
	    stop
	end if

c.. pick up base for filenames, removing ".dat" if necessary ****************

	if(iargc().lt.1) then
		stop 77
	end if
	do ia = 1,iargc()
		call getarg(ia,word)
		if(word(1:1).ne.'-') go to 2
	end do
	stop 76

2	base = word

	do ncbase = 18,1,-1
		if(base(ncbase:ncbase).ne." ") go to 7
	end do
	stop 75

7	if(ncbase.ge.5) then
	    if(base(ncbase-3:ncbase).eq.".dat") then
		base = base(1:ncbase-4)
		ncbase = ncbase-4
	    end if
	end if

	inquire(file=base(1:ncbase)//".dat",exist=qq)
	if(.not.qq) then
		write(0,'("data file not there:",a18)')
     &		base(1:ncbase)//".dat"
		stop
	end if

c.. read data file *************************************************************

	open(5,file=base(1:ncbase)//".dat")

	read(5,*) ndat
	y(0) = 0.0
	read(5,*) (y(i),i=1,ndat)
	read(5,*) leng
	k = 1
	do n = 1,ndat
	    if(y(n).gt.(k*leng)/100.0) then
11		nct(k) = n-1
		k = k+1
		if(y(n).gt.(k*leng)/100.0) go to 11
	    end if
	end do
	do kk = k,100
	    nct(kk) = ndat
	end do
	yav = ndat/leng
c	write(*,'(10i6)') nct

c.. establish parameter values, starting from defaults ************************

	lambda = 3.0
	alpha = 1.0
	beta = leng/ndat
	cc = 2.0
	dd = 2.0
	ee = 1.0
	ff = ndat/leng
	aspr = 2.0
	ac = 0.0
	as = 0.0
	shar = 0.1
	qhar = .false.
	freq = 0.0

	seed = 0
	nsweep = 1000
	nburnin = -1
	nsamp = -1
	q1cp = .false.
	qfix = .false.
	kinit = 0

c.. read control parameters, from files first if present
c.. copy command name for log file

	iline = 0
	call getarg(0,wordl)
	do j = 1,30
	    iline = iline+1
	    line(iline:iline) = wordl(j:j)
	    if(wordl(j:j).eq.' ') go to 54
	end do

c.. get next argument

54	ia = -3
	do while(ia.le.iargc())
	if(ia.eq.-3) then
		inquire(file="pars",exist=qq)
		if(qq) then
			open(3,file="pars")
			ia = -2
		else
			ia = -1
		end if
		go to 50
	else if(ia.eq.-2) then
		read(3,'(a18)',end=51) word
		go to 52
51		close(3)
		ia = -1
		go to 50
	else if(ia.eq.-1) then
		inquire(file=base(1:ncbase)//".par",exist=qq)
		if(qq) then
			ia = 0
		else
			ia = 1
		end if
		go to 50
	else if(ia.eq.0) then
		read(3,'(a18)',end=53) word
		go to 52
53		close(3)
		ia = 1
		go to 50	
	else
		call getarg(ia,word)
		ia = ia+1
	end if

c.. interpret argument

52	if(word(1:3).eq.'-ns') then
		read(word,'(3x,i15)') nsamp
	else if(word(1:3).eq.'-nb') then
		read(word,'(3x,i15)') nburnin
	else if(word(1:2).eq.'-n') then
		read(word,'(2x,i16)') nsweep
	else if(word(1:2).eq.'-a') then
		read(word,'(2x,f16.0)') alpha
	else if(word(1:2).eq.'-b') then
		read(word,'(2x,f16.0)') beta
	else if(word(1:2).eq.'-l') then
		read(word,'(2x,f16.0)') lambda
	else if(word(1:2).eq.'-t') then
		qtab = .true.
	else if(word(1:5).eq.'-seed') then
		read(word,'(5x,i13)') seed
	else if(word(1:5).eq.'-freq') then
		qhar = .true.
		if(word(6:6).eq.'y') then
		    freq = 1.0/365.25
		else
		    read(word,'(5x,f13.0)') freq
		    if(freq.eq.0.0) freq = 1.0
		end if
	else if(word(1:4).eq.'-1cp') then
		q1cp = .true.
	else if(word(1:4).eq.'-fix') then
		qfix = .true.
	else if(word(1:3).eq.'-ki') then
		read(word,'(3x,i15)') kinit
	else if(word(1:2).eq.'-c') then
		read(word,'(2x,f16.0)') cc
	else if(word(1:2).eq.'-d') then
		read(word,'(2x,f16.0)') dd
	else if(word(1:2).eq.'-e') then
		read(word,'(2x,f16.0)') ee
	else if(word(1:2).eq.'-f') then
		read(word,'(2x,f16.0)') ff
	end if

c.. copy argument for log file

	do j = 1,18
	    iline = iline+1
	    line(iline:iline) = word(j:j)
	    if(word(j:j).eq.' ') go to 55
	end do
55	continue

50	continue

	end do

c.. final parameter fixing

	tpf = tp*freq
	if(nburnin.lt.0) nburnin = nsweep/10
	if(nsamp.lt.0) nsamp = max(1,nsweep/1000)
	if(beta.eq.0.0) then
		qbeta = .true.
		beta = 1.0/yav
	else
		qbeta = .false.
	end if
	if(alpha.eq.0.0) then
		qalpha = .true.
		alpha = 1.0
	else
		qalpha = .false.
	end if
	qieee = ieee_chk("before abcon")
	logabcon = alpha*log(beta)-real(dlgama(dble(alpha)))
	qieee = ieee_chk("after abcon")

	call sdrni(seed)

c.. sort out filename stuff ****************************************************

	ijk = 1

1	istd = 6-int(log10(0.5+ijk))
	write(num,'(i6)') ijk

	inquire(file=base(1:ncbase)//sep//num(istd:6)//".log",exist=qq)
	if(qq) then
		ijk = ijk+1
		go to 1
	end if

	prfx = base(1:ncbase)//sep//num(istd:6)
	npf = ncbase+8-istd

	open(3,file=prfx(1:npf)//".log")
	open(7,file=prfx(1:npf)//".ht")
	open(8,file=prfx(1:npf)//".k")
	open(9,file=prfx(1:npf)//".pos")
	open(10,file=prfx(1:npf)//".av")
	if(qbeta) open(11,file=prfx(1:npf)//".beta")
	if(qalpha) open(12,file=prfx(1:npf)//".alpha")
	if(q1cp) open(13,file=prfx(1:npf)//".1cp")
	if(qhar) open(14,file=prfx(1:npf)//".har")

c.. log basic parameters

	do lun = 0,3,3
	  write(lun,*)
	  write(lun,*) prfx(1:npf),": ",line(1:iline)
	  write(lun,*) "random number seed:",seed
	  if(qalpha) then
	    write(lun,'(/"alpha     Gamma:",2f8.2)') cc,dd
	  else
	    write(lun,'(/"alpha   ",f8.4)') alpha
	  end if
	  if(qbeta) then
	    write(lun,'("beta      Gamma:",f8.2,f12.6)') ee,ff
	  else
	    write(lun,'("beta    ",f8.4)') beta
	  end if
	  write(lun,'("lambda  ",f8.4)') lambda
	  write(lun,'("seed    ",i10)') seed	
	  write(lun,'("nsweep  ",i10)') nsweep
	  write(lun,'("nburnin ",i10)') nburnin
	end do

c.. prior distribution on number of steps **************************************

	qieee = ieee_chk("before prior")
	p(0) = exp(-lambda)
	do k = 1,kmax
	    p(k) = p(k-1)*lambda/k
	end do
c	write(*,'(10f8.4)') (p(k),k=0,kmax)

c.. birth and death rates

	if(qfix) then

	    do k = 0,kmax
		b(k) = 0.0
		d(k) = 0.0
	    end do

	else if(qoptbd) then

	    c = min(1.0/min(1,p(1)/p(0)),1.0/min(1,p(kmax-1)/p(kmax)))
	    do k = 1,kmax-1
	        c = min(c,1.0/(min(1.0,p(k+1)/p(k))+min(1.0,p(k-1)/p(k))))
	    end do
	    c = 0.9*c
	    d(0) = 0.0
	    do k = 0,kmax-1
	        b(k) = c*min(1.0,p(k+1)/p(k))
	        d(k+1) = c*min(1.0,p(k)/p(k+1))
	    end do
	    b(kmax) = 0.0

	else

	    do j = 0,kmax
	        b(j) = 0.4
	        d(j) = 0.4
	    end do
	    d(0) = 0.0
	    b(kmax) = 0.0

	end if

	ndacc = 0
	ndtry = 0
	nbacc = 0
	nbtry = 0
	nhacc = 0
	nhtry = 0
	npacc = 0
	nptry = 0
	nalacc = 0
	naltry = 0
	nharacc = 0
	nhartry = 0

	do ig = 1,ng
	    t(ig) = (ig-0.5)*leng/ng
	    xav(ig) = 0.0
	end do

	k = kinit
	do j = 0,k+1
	    sp(j) = real(j)*leng/real(k+1)
	end do
	do j = 0,k
	    sh(j) = yav
	end do

	avk = 0.0
	avalpha = 0.0
	avbeta = 0.0
	avac = 0.0
	avas = 0.0

c	write(*,'("start loop")')

c.. start of main loop ************************************************************

	qieee = ieee_chk("before main loop")

	do sweep = 1,nburnin+nsweep

	usw = sdrand()

	if(usw.lt.d(k)) then

c.. death

		kill = 1+int(k*sdrand())
		w1 = sp(kill)-sp(kill-1)
		w2 = sp(kill+1)-sp(kill)
		h1 = sh(kill-1)
		h2 = sh(kill)
		h = h1**(w1/(w1+w2))*h2**(w2/(w1+w2))
		u = 1.0/(1.0+(h2/h1)**(1.0/power))
		
c.. likelihood term

		call likrat(sp(kill-1),sp(kill),sp(kill+1),
     &			h,h1,h2,ac,as,tpf,logratio)
		
c.. prior term

		logratio = logratio+log(p(k)/p(k-1))
     &			+log((2.0*k)*(2.0*k+1.0)/leng**2)
     &			+log(w1*w2/(w1+w2))
     &			+logabcon+(alpha-1.0)*log(h1*h2/h)
     &			-beta*(h1+h2-h)

c.. proposal term

		logratio = logratio+log(d(k)*leng/(b(k-1)*k))

c.. Jacobian

		logratio = logratio+log(power*h1*h2/(h*u*(1.0-u)))
		
		logratio = -logratio
		ratio = exp(max(-30.0,min(0.0,logratio)))
		
		if(sdrand().lt.ratio) then
			sh(kill-1) = h
			do j = kill,k-1
			    sh(j) = sh(j+1)
			    sp(j) = sp(j+1)
			end do
			sp(k) = leng
			k = k-1
			if(sweep.gt.nburnin) ndacc = ndacc+1
		end if
		if(sweep.gt.nburnin) ndtry = ndtry+1

		qieee = ieee_chk("after death")

	else if(usw.lt.d(k)+b(k)) then

c.. birth

		pos = leng*sdrand()
		do join = 1,k+1
		    if(sp(join).gt.pos) go to 6
		end do
		stop 99
6		w1 = pos-sp(join-1)
		w2 = sp(join)-pos
		h = sh(join-1)
		u = sdrand()
		h1 = h*(u/(1.0-u))**(power*w2/(w1+w2))
		h2 = h1*((1.0-u)/u)**power
		
		qieee = ieee_chk("after uj")

c.. likelihood term

		call likrat(sp(join-1),pos,sp(join),h,h1,h2,ac,as,tpf,logratio)

c.. prior term

		logratio = logratio+log(p(k+1)/p(k))
     &			+log((2.0*k+2.0)*(2.0*k+3.0)/leng**2)
     &			+log(w1*w2/(w1+w2))
     &			+logabcon+(alpha-1.0)*log(h1*h2/h)
     &			-beta*(h1+h2-h)

c.. proposal term

		logratio = logratio+log(d(k+1)*leng/(b(k)*(k+1)))

c.. Jacobian

		logratio = logratio+log(power*h1*h2/(h*u*(1.0-u)))
		
		qieee = ieee_chk("after ratio")
		if(qieee) then
			write(*,*) pos,join,w1,w2,h1,h2
			write(*,*) p(k),p(k+1),alpha,beta,b(k),d(k+1),logabcon
			write(*,*) ratlik,logratio
			stop
		end if

		ratio = exp(max(-30.0,min(0.0,logratio)))

		if(sdrand().lt.ratio) then
			sp(k+2) = leng
			do j = k,join,-1
			    sp(j+1) = sp(j)
			    sh(j+1) = sh(j)
			end do
			sp(join) = pos
			sh(join-1) = h1
			sh(join) = h2
			k = k+1
			if(sweep.gt.nburnin) nbacc = nbacc+1
		end if

		if(sweep.gt.nburnin) nbtry = nbtry+1

		qieee = ieee_chk("after birth")
	end if

c.. change a step height

		kch = int((k+1)*sdrand())
		shnew = sh(kch)*exp(conh*(sdrand()-0.5))

		call likrat(sp(kch),sp(kch+1),sp(kch+1),
     &			sh(kch),shnew,shnew,ac,as,tpf,logratio)

		logratio = logratio+alpha*(log(shnew/sh(kch)))
     &			-beta*(shnew-sh(kch))

		ratio = exp(max(-30.0,min(0.0,logratio)))
		
		if(sdrand().lt.ratio) then
			sh(kch) = shnew
			if(sweep.gt.nburnin) nhacc = nhacc+1
		end if

		if(sweep.gt.nburnin) nhtry = nhtry+1
		qieee = ieee_chk("after ht")
		if(qieee) then
		write(*,*) shnew,sh(kch),ratio,logratio,sweep
		stop
		end if

c... move a step

	    if(k.ne.0) then

		kmove = 1+int(k*sdrand())
		spnew = sp(kmove-1)+sdrand()*(sp(kmove+1)-sp(kmove-1))

		if(spnew.lt.sp(kmove)) then
		    call likrat(spnew,sp(kmove),sp(kmove),
     &			sh(kmove-1),sh(kmove),sh(kmove),ac,as,tpf,logratio)
		else
		    call likrat(sp(kmove),spnew,spnew,
     &			sh(kmove),sh(kmove-1),sh(kmove-1),ac,as,tpf,logratio)
		end if

		logratio = logratio+
     &			log(((sp(kmove+1)-spnew)/(sp(kmove+1)-sp(kmove)))
     &			*((spnew-sp(kmove-1))/(sp(kmove)-sp(kmove-1))))

		ratio = exp(max(-30.0,min(0.0,logratio)))

		if(sdrand().lt.ratio) then
			sp(kmove) = spnew
			if(sweep.gt.nburnin) npacc = npacc+1
		end if

		if(sweep.gt.nburnin) nptry = nptry+1
		qieee = ieee_chk("after pos")

	    end if

c.. update beta

		if(qbeta) then

		    hsum = 0.0
		    do j = 0,k
			hsum = hsum+sh(j)
		    end do

		    beta = rgamma(ee+alpha*(k+1))/(ff+hsum)
		    logabcon = alpha*log(beta)-real(dlgama(dble(alpha)))

		    qieee = ieee_chk("after beta")

		    if(qieee) then
		    write(*,*) alpha,beta,k,hsum,ee,ff
		    end if

		end if

c.. update alpha

		if(qalpha) then

		    prodh = 1.0
		    do j = 0,k
			prodh = prodh*sh(j)
		    end do

		    alphaN = alpha*aspr**(sdrand()-0.5)

		    logratio = (alphaN-alpha)*
     &			((k+1)*log(beta)-dd+log(prodh))
     &			+(k+1)*(dlgama(dble(alpha))-dlgama(dble(alphaN)))
     &			+log(alphaN/alpha)*cc

		    logratio = max(-30.0,min(0.0,logratio))

		    if(sdrand().lt.exp(logratio)) then
			alpha = alphaN
			logabcon = alpha*log(beta)-real(dlgama(dble(alpha)))
			if(sweep.gt.nburnin) nalacc = nalacc+1
		    end if

		    if(sweep.gt.nburnin) naltry = naltry+1

		    qieee = ieee_chk("after alpha")

		end if

c.. update harmonic parameters

		if(qhar.and.mod(sweep,25).eq.0) then

		    acN = ac+shar*(sdrand()-0.5)
		    asN = as+shar*(sdrand()-0.5)

		    if(acN**2+asN**2.lt.1.0) then

		    logratio = 0.0
		    do i = 1,ndat
			logratio = logratio+log((1.0+acN*cos(tpf*y(i))+asN*sin(tpf*y(i)))/
     &				(1.0+ac*cos(tpf*y(i))+as*sin(tpf*y(i))))
		    end do

		    do j = 0,k
			logratio = logratio-sh(j)*
     &				((acN-ac)*(sin(tpf*sp(j+1))-sin(tpf*sp(j)))
     &				-(asN-as)*(cos(tpf*sp(j+1))-cos(tpf*sp(j))))/tpf
		    end do

		    logratio = max(-30.0,min(0.0,logratio))

		    if(sdrand().lt.exp(logratio)) then
			ac = acN
			as = asN
			if(sweep.gt.nburnin) nharacc = nharacc+1
		    end if

		    end if

		    if(sweep.gt.nburnin) nhartry = nhartry+1

		end if

c.. end of sweep: report

	if(sweep.gt.nburnin) then

		if(mod(sweep-nburnin,nsamp).eq.0) then
		    write(9,*) (sp(j),j=1,k)
		    write(7,*) (sh(j),j=0,k)
		    write(8,*) k
		    if(qbeta) write(11,*) beta
		    if(qalpha) write(12,*) alpha
		    if(q1cp.and.k.eq.1) write(13,*) sp(1),sh(0),sh(1)
		    if(qhar.and.mod(sweep,25).eq.0) write(14,*) ac,as
		end if

		istp = 0
		do ig = 1,ng
152		    if(sp(istp+1).gt.t(ig)) go to 153
		    istp = istp+1
		    go to 152
153		    x(ig) = sh(istp)
		    xav(ig) = xav(ig)+x(ig)
		end do

		avk = avk+k
		if(qalpha) avalpha = avalpha+alpha
		if(qbeta) avbeta = avbeta+beta
		if(qhar) then
		    avac = avac+ac
		    avas = avas+as
		end if

		if(qtab) write(4,'(10f6.3)') x

	end if

c	write(0,'(i6)') sweep,k
c	write(0,*) (sh(ih),ih=0,k)

c.. end of main loop
	end do

c.. compute means

	avk = avk/nsweep
	if(qalpha) avalpha = avalpha/nsweep
	if(qbeta) avbeta = avbeta/nsweep
	if(qhar) then
	    avac = avac/nsweep
	    avas = avas/nsweep
	end if

c.. summarise MCMC performance

	nacc = ndacc+nbacc+nhacc+npacc+nalacc+nharacc
	do lun = 0,3,3
	  write(lun,'(/"         acc    try")')
  	  write(lun,'("death",i7,i7)') ndacc,ndtry
	  write(lun,'("birth",i7,i7)') nbacc,nbtry
	  write(lun,'("ht   ",i7,i7)') nhacc,nhtry
	  write(lun,'("pos  ",i7,i7)') npacc,nptry
	  if(naltry.gt.0) write(lun,'("alpha",i7,i7)') nalacc,naltry
	  if(nhartry.gt.0) write(lun,'("harm ",i7,i7)') nharacc,nhartry
	  write(lun,'(/"total acceptances",i7/)') nacc
	  write(lun,'("mean k      ",f8.4)') avk
	  if(qalpha) write(lun,'("mean alpha ",g13.5)') avalpha
	  if(qbeta) write(lun,'("mean beta",g13.5)') avbeta
	  if(qhar) then
	    write(lun,'("mean ac   ",g13.5)') ac
	    write(lun,'("mean as   ",g13.5)') as
	  end if
	  write(lun,*)
	end do

c.. output mean curve

	do ig = 1,ng
	    xav(ig) = xav(ig)/nsweep
	end do

	write(10,*) xav

	stop

	end

c*************************************************

	subroutine likrat(p1,p2,p3,h,h1,h2,ac,as,tpf,logratio)

	parameter(nmax=500)
	real leng,logratio
	logical qieee,ieee_chk
	dimension y(0:nmax),nct(100)
	common/datblk/y,nct,leng

	qieee = ieee_chk("in likrat1")

	n1 = nct(min(100,1+int(100.0*p1/leng)))
	do while(y(n1).gt.p1)
	    n1 = n1-1
	end do

	n2 = nct(min(100,1+int(100.0*p2/leng)))
	do while(y(n2).gt.p2)
	    n2 = n2-1
	end do

	qieee = ieee_chk("in likrat2")

	cdiff = (p2-p1)

	if(tpf.ne.0.0) then
	    cdiff = cdiff+(ac*(sin(tpf*p2)-sin(tpf*p1))
     &		-as*(cos(tpf*p2)-cos(tpf*p1)))/tpf
	end if

	logratio = (n2-n1)*log(h1/h)-(h1-h)*cdiff

	qieee = ieee_chk("in likrat3")

	if(p2.ne.p3) then

	    n3 = nct(min(100,1+int(100.0*p3/leng)))
	    do while(y(n3).gt.p3)
		n3 = n3-1
	    end do

	    cdiff = (p3-p2)

	    if(tpf.ne.0.0) then
		cdiff = (p3-p2)+(ac*(sin(tpf*p3)-sin(tpf*p2))
     &		-as*(cos(tpf*p3)-cos(tpf*p2)))/tpf
	    end if

	    logratio = logratio+(n3-n2)*log(h2/h)-(h2-h)*cdiff

	end if

	qieee = ieee_chk("in likrat4")

	return

	end

c******************
